/*
 *      Program do przegladania logow z ekg.
 *
 *
 *      copyright (c) 2003 by Lukasz Bartnik
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __CMD_COMPLETE_HH__
#define __CMD_COMPLETE_HH__


#include "ui-commands.h"
#include "texts.h"

/**
 * @brief Komenda quit
 */
class cmd_complete_t : public command_ui_base_t {
public:

	cmd_complete_t() :  command_ui_base_t("key_tab"),
			_first_time(true)
	{
	}

	void run(const std::vector<std::string> & argv)
	{
		/// zeby moc sprawdzic, czy dwa razy pod rzad byl tab
		key_pair tabs(0x09, 0x09);

		if(_first_time) {
			build_tables();
			_first_time = false;
		}

		if (!_cmdline->is_good_to_complete())
			return;

		std::string arg = _cmdline->get_last();
		std::string cmd = _cmdline->get_first();
 
		// jesli cmd jest pelna komenda, to trzeba szukac jej argumentow
		std::map<std::string, command_base_t* >::iterator i;
		i = _cmds->find(cmd);

		// true, jesli jest wiecej niz jeden czlon w linii komend
		bool single = _cmdline->one_piece();

		if (single && i==_cmds->end()) {
			complete_command(cmd);
			if (_match.size() == 1) {
				std::string tmp = _match.front().substr(
						cmd.size(), std::string::npos);
				if (tmp.size())
					_cmdline->add_completition(tmp);
			}

			if (_match.size() > 1 && _cmdline->get_keys() == tabs)
				print_matching();
		} else {
			if (single)
				arg.clear();

			complete_argument(cmd, arg);
			if (_match.size() == 1) {
				std::string tmp = _match.front().substr(
						arg.size(), std::string::npos);
				if (tmp.size())
					_cmdline->add_completition(tmp);
			}

			if (_match.size() > 1 && _cmdline->get_keys() == tabs)
				print_matching();
		}


		//_info->print("Pierwszy: "+cmd+", ostatni: "+arg, ALL_INFO_FMT);
	} // run()

private:

	/// buduje tablice dopelniania pierwszego poziomu
	void build_tables()
	{
		for(std::map<std::string, command_base_t*>::iterator i=_cmds->begin();
			i!=_cmds->end(); ++i)
		{
			const std::string & str = i->second->completition_lvl1();
			if (str.size())
				_lvl1.push_back(str);
		}
	}

	/// dopelnianie komendy, nie argumentu
	void complete_command(const std::string & str)
	{
		_match.clear();
		for(std::list<std::string>::iterator i=_lvl1.begin();
			i!=_lvl1.end(); ++i)
			if(i->substr(0, str.size()) == str)
				_match.push_back(*i);
	}

	/// dopelnia argumenty
	void complete_argument(const std::string & cmd, const std::string & arg)
	{
		_match.clear();

		std::map<std::string, command_base_t* >::iterator i;
		if ((i = _cmds->find(cmd)) != _cmds->end()) {
			const std::vector<std::string> & v = i->second->completition_lvl2();

			for(std::vector<std::string>::const_iterator i=v.begin();
				i!=v.end(); ++i)
				if(i->substr(0, arg.size()) == arg)
					_match.push_back(*i);
		}
	}

	/// wypisuje pasujace stringi (z listy _match)
	void print_matching()
	{
		std::string tmp;
		unsigned int largest = 0;
		std::string spaces = "                                         ";

		for(std::list<std::string>::iterator i=_match.begin();
			i!=_match.end(); ++i)
			if (largest < i->size())
				largest = i->size();

		largest += 2;

		unsigned int cols = _info->hsize() / largest;
		if (cols > _match.size())
			cols = _match.size();

		unsigned int lastrow = _match.size() % cols;

		std::vector<std::list<std::string>::iterator> col;
		std::list<std::string>::iterator mi = _match.begin();

		for (unsigned int i=0; i<cols; ++i) {
			unsigned int size = _match.size()/cols + (lastrow != 0);
			col.push_back(mi);
			while (size--)
				++mi;
			
			if (lastrow)
				--lastrow;
			
		}

		while (col.back() != _match.end())
		{
			for (unsigned int i=0; i<col.size() && col[i] != _match.end(); ++i) {
				tmp += *col[i];
				tmp += spaces.substr(0,largest - col[i]->size());
				++col[i];
			}

			tmp += '\n';
		}

		_info->print(tmp, "");
	}

	/// czy dopelnianie jest odpalane pierwszy raz
	bool _first_time;
	/// tutaj trzymamy wyniki przeszukiwania
	std::list<std::string> _lvl1, _match;

};

#endif /* __CMD_COMPLETE_HH__ */
