/*	      Directory class for reading directory entries on linux
 *	      Copyright (C) 2003, Piotr Kalinowski <pitkali@tlen.pl>
 *	      All rights reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to permit
 * persons to whom the Software is furnished to do so, subject to the
 * following conditions:
 * 
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#ifndef _CPP_DIRECTORY
#define _CPP_DIRECTORY 1

#include <sys/types.h>
#include <string>

class _directory; // forward declaration of implementation class

// Here comes the interface to directories. For the time being it supports only
// dirent.h known from linux. Additions, which should be placed into implementation
// file, are wellcome.
class Directory {
  public:
    Directory();
    Directory(const std::string& s);
    ~Directory();

    // Copy constructor and assignment operator.
    Directory(const Directory&);
    Directory& operator=(const Directory&);
    bool operator==(const Directory&);
    
    bool open(const std::string& s); // return false on error
    void close(void);
    bool isopen(void) const; // check if any directory is opened
    bool isbad(void) const; // check if there were any errors
    
    std::string read_entry(void) const; // read directory entry, empty string for eof
    off_t tell(void) const; // get current stream position
    void seek(off_t) const; // set current position
    void rewind(void) const; // reset position
    
    const std::string& get_error(void) const; // last error info
    const std::string& get_name(void) const;
  private:
    _directory* impl; // implementation handle
};

#endif
