/*
 *      Program do przegladania logow z ekg.
 *      Glowny plik programu.
 *
 *
 *      copyright (c) 2003 by Lukasz Bartnik
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include "ui.h"
#include "options.h"
#include "environment.h"
#include "errors.h"

#include <iostream>
#include "ekglog.hpp"
using namespace EkgLog;

int main(int argc, char *argv[])
{
	check_options_t options(argc, argv);
	dBase db;

	// jesli nastapil blad okreslania srodowiska
	if (!env)
		error_exit(ENV_ERROR);

	if (options.show_help()) {
		std::cout << CMDLINE_HELP << "\n";
		exit(0);
	}

	if (options.show_nicks()) {
		const userlist_t & u = env.get_userlist();
		std::cout << "Lista " << u.size() << " nickow:\n";
		for (userlist_t::const_iterator i=u.begin(); i!=u.end(); ++i)
			std::cout << i->first << " " << i->second << "\n";

		std::cout << std::endl;
	}

	// przygotowanie bazy danych
	if (!options.no_dbase()) {
		dBase::log_format_t fmt;
		const std::string & log = env.get_variable("log");
		if (log == "1")
			fmt = dBase::FILE;
		else if (log == "2")
			fmt = dBase::DIR;
		else if (log == "5")
			fmt = dBase::FILEGZ;
		else if (log == "6")
			fmt = dBase::DIRGZ;
		else
			error_exit(LOGFORMAT_ERROR);

		db.open(env.get_variable("log_path"), fmt, env.get_userlist());
	}

	if (!options.no_ui()) {
		ui_t ui(db);
		ui.loop();
	}

	return 0;
}
