/*
 *      Program do przegladania logow z ekg.
 *      Odpalanie i konczenie ncurses.
 *
 *      copyright (c) 2003 by Lukasz Bartnik
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "ui.h"

// komendy
#include "help.h"
#include "quit.h"
#include "echo.h"
#include "about.h"
#include "complete.h"
#include "merge.h"
#include "stats.h"
#include "find.h"
#include "dump.h"
#include "dellog.h"

#include "texts.h"


ui_t::ui_t(dBase & db)
	: _list_width(20),
	  _main_width(COLS-_list_width),_main_height(LINES-2),
	  _nicks_height((LINES-2)/2),_chats_height(LINES-2-_nicks_height),

	  nc_main(newwin(_main_height, _main_width,0,0)),
	  nc_nicks(newwin(_nicks_height, _list_width, 0, COLS-_list_width)),
	  nc_chats(newwin(_chats_height, _list_width, _nicks_height, COLS-_list_width)),
	  nc_cmdline(newwin(1, COLS, LINES-1, 0)),
	  nc_status(newwin(1, COLS, LINES-2, 0)),

	  _main(WINDOW_TITLE_MAIN, COLS-_list_width, LINES-2, 1000, nc_main),
	  _info(WINDOW_TITLE_INFO, COLS-_list_width, LINES-2, 1000, nc_main),
	  _nicks(WINDOW_TITLE_NICKS, _list_width, (LINES-2)/2, nc_nicks),
	  _chats(WINDOW_TITLE_CHATS, _list_width, LINES-2 - (LINES-2)/2, nc_chats),

	  _cmdline(COLS, nc_cmdline),
	  _status(COLS, nc_status),
	  _end_program(false),

	  _db(db)
{
	_info.set_displayed(true);
	_info.print(INFO_WELCOME, INFO_WELCOME_FMT);


	// rejestrujemy klawisze i kombinacje specjalne - sterowanie ui
	// _cmdline.register_key(0x09,      "key_tab");

	command_base_t * cptr;

	// Dodajemy komendy do mapy sterowania ui.
	#define reg_key(key, command) cptr = new command();\
				_cmds[cptr->name()] = cptr;\
				_cmdline.register_key(key, cptr->name());
	#define reg_seq(key1, key2, command) cptr = new command();\
				_cmds[cptr->name()] = cptr;\
				_cmdline.register_sequence(key1, key2, cptr->name());

	reg_key(KEY_F(1), uicmd_f1_t)
	reg_key(KEY_F(2), cmd_help_t)
	reg_key(KEY_F(10), cmd_quit_t)
	reg_key(KEY_UP, uicmd_up_t)
	reg_key(KEY_DOWN, uicmd_down_t)
	reg_key(KEY_PPAGE, uicmd_pgup_t)
	reg_key(KEY_NPAGE, uicmd_pgdown_t)
	reg_key(0x09, cmd_complete_t)
	reg_key(0x0A, uicmd_enter_t)
	reg_seq(0x1B, '1', uicmd_alt1_t)
	reg_seq(0x1B, '2', uicmd_alt2_t)
	reg_seq(0x1B, '3', uicmd_alt3_t)
	reg_seq(0x1B, '4', uicmd_alt4_t)
	reg_seq(0x1B, '5', uicmd_alt5_t)

	#undef reg_key
	#undef reg_seq

	// zwykle komendy rejestrujemy normalnie
	cptr = new cmd_echo_t();
	_cmds[cptr->name()] = cptr;

	cptr = new cmd_about_t();
	_cmds[cptr->name()] = cptr;

	cptr = new cmd_merge_t();
	_cmds[cptr->name()] = cptr;

	cptr = new cmd_stats_t();
	_cmds[cptr->name()] = cptr;

	cptr = new cmd_find_t();
	_cmds[cptr->name()] = cptr;

	cptr = new cmd_dump_t();
	_cmds[cptr->name()] = cptr;

	cptr = new cmd_dellog_t();
	_cmds[cptr->name()] = cptr;


	// ustawiamy wskazniki tak, aby komendy mogly sie odwolywac do okien
	command_base_t::_info = &_info;
	command_base_t::_main = &_main;
	command_base_t::_nicks = &_nicks;
	command_base_t::_chats = &_chats;
	command_base_t::_cmds = &_cmds;
	command_base_t::_cmdline = &_cmdline;
	command_base_t::_status = &_status;
	command_base_t::_end_program = &_end_program;
	command_base_t::_active_window = _active_window = 0;
	command_base_t::_db = &_db;
	command_base_t::_ui = this;

	set_lists();

	// statystyki logow
	std::string stats = STATS_PART1 + text_cast(_db.messages());
	stats += STATS_PART2 + text_cast(_db.statuses());
	stats += STATS_PART3 + text_cast(_db.time());
	stats += STATS_PART4;
	_info.print(stats);
}


ui_t::~ui_t()
{
}

void ui_t::set_lists()
{
	_nicks.clear();
	_chats.clear();

	// ustawiamy tresc list
	std::map<std::string, uin_t> m;
	for (log_iterator i=_db.begin(); i!=_db.end(); ++i) {
		if (i->second.nick().size())
			m[i->second.nick()] = i->second.uin();
		else
			m[text_cast(i->second.uin())] = i->second.uin();
	}

	if (m.size() == 0)
		return;

	for (std::map<std::string, uin_t>::iterator i=m.begin(); i!=m.end(); ++i)
		_nicks.append(i->first, i->second);

	log_iterator log = _db.log(_nicks.get_chosen_value());
	chat_reverse_iterator i=log->second.rbegin();
	for (; i!=log->second.rend(); ++i) {
		if (i->second.begin() == i->second.end())
			continue;

		_chats.append(format_date(i->first), i->first);
	}

	// przy zmianie zawartosci list trzeba tez odnowic listy uzupelniania
	std::map<std::string, command_base_t*>::iterator cmditer;
	if ( (cmditer = _cmds.find("dellog")) != _cmds.end() ) {
		cmd_dellog_t * d = dynamic_cast<cmd_dellog_t* > (cmditer->second);
		d->complete();
	}
}


void ui_t::loop()
{
	std::string cmd;
	std::map<std::string, command_base_t*>::iterator cmditer;

	// aktywna jest linia komend - lepiej, zeby ta komenda byla
	// juz zarejestrowana...
	_cmds["alt_5"]->run(std::vector<std::string>());

	while(!_end_program) {
		refresh();

		try {
		    cmd = _cmdline.get_cmd();
		}
		// linia komend moze wyrzucic jakis blad
		catch(cmdline_t::cmdline_error_t & e) {
			if(e.type == cmdline_t::PARSE_ERROR) {
				std::string tmp(ERROR_CMDLINE_1);
				tmp += e.ch;
				if(e.ch == '\\')
					tmp += '\\';

				_info.print(tmp, ALL_ERROR_FMT);
			}
			else if(e.type == cmdline_t::WRONG_END)
				_info.print(ERROR_CMDLINE_2, ALL_ERROR_FMT);
			else
				_info.print(ERROR_CMDLINE_3, ALL_ERROR_FMT);

			continue;
		}

		if ((cmditer = _cmds.find(cmd)) != _cmds.end())
			cmditer->second->run(_cmdline.argv());
		else
			_info.print(ERROR_CMDLINE_4_B+cmd+ERROR_CMDLINE_4_E, ALL_ERROR_FMT);
	}
}


void ui_t::refresh()
{
	_main.print();
	_info.print();
	_nicks.print();
	_chats.print();

	// ustawiamy tresc paska stasu
	_status.set_more(   (_main.is_displayed() && _main.more()) ||
			    (_info.is_displayed() && _info.more())
			);
	// teraz ustwiamy info o aktywnych lecz niewidocznych oknach
	_status.set_active_info( (!_info.is_displayed() && _info.has_new())
				    ? WINDOW_TITLE_INFO : "" );
	_status.set_active_main( (!_main.is_displayed() && _main.has_new())
				    ? WINDOW_TITLE_MAIN : "" );
	_status.print();

	// linia komend musi byc wypisana na koncu, zeby
	// kursor byl zostawiony w niej
	_cmdline.print();

	doupdate();
}
