/*
 *      Program do przegladania logow z ekg.
 *      Klasa okna - listy wyboru.
 *
 *
 *      copyright (c) 2003 by Lukasz Bartnik
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __WINLIST_HH__
#define __WINLIST_HH__


#include "window.h"


class winlist_t : public window_t {
public:
	winlist_t(const char *title, 
		int width, int height,
		WINDOW *win=0);

	virtual ~winlist_t() {}

	/// Przeladowana - dodane linie boczne.
	virtual void print();
	/// Przesuwanie zawartosci okna.
	virtual void moveup();
	virtual void movedown();
	virtual void movepgup();
	virtual void movepgdown();

	/**
	 * @brief Dodanie na koniec listy wyboru nowej pozycji.
	 *
	 * W tresci wpisu nie moze byc znaku nowej linii.
	 */
	void append(const std::string & str, int value);

	/// Czysci okno.
	void clear();
	/// Zwraca wartosc/tresc wybranej linii.
	int get_chosen_value();
	/// Zwraca tresc wybranej pozycji
	const std::string & get_chosen_str();
	/// Ustawia pozycje zawierajaca podana wartosc jako aktywna.
	void set_active_val(int val);

	/// Interfejs do przegldania listy w kolejnosci wystepowania w oknie - wartosci.
	typedef std::vector<int>::const_iterator const_iterator;
	
	const_iterator begin() const;
	const_iterator end() const;
	const_iterator find(int val);

	/// Przegladanie listy wyswietlanej w oknie.
	typedef std::vector<line_t>::const_iterator const_str_iterator;

	const_str_iterator str_begin() const;
	const_str_iterator str_end() const;
	const_str_iterator str_find(std::string val);

protected:

	/// Ustawia wybrany element listy jak najblizej srodka ekranu.
	void set_screen_start();

	/// numer wybranej linii
	unsigned int _chosen_line;
	/// wartosci przypisane kazdej z linii zwracane przy jej wybraniu
	std::vector<int> _line_values;
	/// czesc tytulu okna do sklejenia z liczba pozycji
	std::string _title_part;
};

#endif /* __WINLIST_HH__ */
